%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Dynamics of ball flying through the air.
%
% The intention of this script is work out the basic structure of how to 
% trace the motion of an object through the air. In this particular case 
% we are seeking to examine the behavior of a ball as it falls through
% the air.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear 'all';                   % tells FreeMat to 'forget' previous variable assignments, clean the board!

c1 = [.5 .4 .7];               % defines custom color 1
c2 = [.5 .5 .5];               % defines custom color 2


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Defining useful variables
%
% The general plan is to allow the ball to fall incrementally
% from some height and figure out what it does.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gravcnst = 10.0;     % assume graviational constant in m/s/s, + is down
dragcnst = 0.30;     % drag constant, might use it later
angle = 30.0;          % angle of launch
initvelocity = 5.0;   % initial velocity in m/s



numpts = 300;         % total number of times to increment
timestep = 0.01;      % increment time step in s

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Initialize ball position array....
% Because we are taking a classic interative approach to where
% how the ball moves through the world we need to have an array
% which keeps track of where the ball is each time we calculate
% a new position and velocity. There needs to be an entry in the
% array available for each of the pts that we calculate (numpts) 
% defined above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i = (1:numpts);
    ballx(i) = 0;         % initialize y postions to 0 
    bally(i) = 0;         % initialize y postions to 0 
    dragforce(i) = 0;     % initialize drag forces to 0
    ballvelocx(i) = 0;  % initialize ball x velocity
    ballvelocy(i) = 0;  % initialize ball y velocity
    time(i) = (i-1)*timestep;  % time data points
end

ballx(1) = 0.0;       % choose coordinate axis at the ball and + down
bally(1) = 2.5;       % choose coordinate axis at the ball and + down

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This is an example of how to set up a point that you want to
% plot on the figure for what ever reason.....
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

specialx = 3.5;
specialy = 0.6;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Where is the ball....analytically
%
% The location of the ball is determined from the 
% 4 magic kinematic equations.
%
% mathtime is the times at which I will calculate the x an y positions
% mathxposition, mathyposition are the caculated positions from the 4 magic
% equations. Note the  ".^" exponential operator which is required because 
% mathtime is a vector and FreeMat needs to know which of several possible ways
% should it use to square it. This operator ".^" squares each individual term in
% the vector.
% 
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


mathtime = 0:.01:1.0;
mathxposition=ballx(1)+(initvelocity*cosd(angle))*mathtime;
mathyposition=bally(1)+(initvelocity*sind(angle))*mathtime+0.5*(-gravcnst)*mathtime.^2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Where is the ball....interatively (for the time being I am just showing
% that this method leads to the same prediction as our analytic approach)
%
% The location of the ball is determined by assuming the acceleration
% is constant for a short period of time (this is how we deal with air drag)
% and then the acceleration is recalculated at the end of each small interval
% of time. The 4 magic kinematic equations apply for the duration of the small
% moment of time (timestep)
% 
% First we have to define our starting point for the interative model which is 
% below.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

currentaccel(1) = -gravcnst;  % take acceleration = g
ballvelocx(1) = initvelocity*cosd(angle);   % assumed at x velocity begining in m/s
ballvelocy(1) = initvelocity*sind(angle);   % assumed at x velocity begining in m/s

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Where is the ball....
%
% The location of the ball is determined interatively from the 
% 4 magic kinematic equations. Much of this would need to be modified to
% include air drag. You can look at the balldrop.m file to see what changes
% would be needed.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j = [2:numpts]
    ballx(j) = ballx(j-1) + ballvelocx(j-1)*timestep; % standard horizontal behavior is negative
    ballvelocx(j) = ballvelocx(j-1);
    bally(j) = bally(j-1) + ballvelocy(j-1)*timestep +0.5*currentaccel(j-1)*(timestep)^2; % standard horizontal behavior is negative
    ballvelocy(j) = ballvelocy(j-1)+currentaccel(j-1)*timestep;
    currentaccel(j) = -gravcnst;    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% A quick check to make sure I entered the data points that I intended
% and that there are the same number of x and y values:) WATCH OUT -- 
% labels for numpts are easily confused with the timesteps up above!!
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%disp(x,y);                                % this  allow me to display my values to check
numxpts=length(ballx);                         % number of x data points
numypts=length(bally);                        % number of y data points

if (numxpts == numypts)  % if there are equal numbers of x and y data pts then confirm # of data points
    printf('There are %f  data points\n',numxpts);
else                     % if data points are matched up issue a warning
    printf('WARNING: YOUR DATA ARRAYS ARE UNEQUAL!\n');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% making the plot....
% 
% the markerfacecolor just colors in the square which marks the 
% interative model.
%
% The 'figure() command allows me to open and modify two different plot
% at the same time.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1);
ballplot = plot(ballx,bally,'gs',specialx,specialy,'ks',mathxposition,mathyposition,'b-');
%set(ballplot,'markerfacecolor','g'); % if you want to fill the squares in with color

figure(2);
detailplot = plot(ballx,bally,'gs',mathxposition,mathyposition,'b-');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Set up overall plot ranges....
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plotminx = 0.;   
plotminy = 0.;
plotmaxx = 4.5;
plotmaxy = 3.5;

figure(1);
axis([plotminx,plotmaxx,plotminy,plotmaxy]);  %  set limits on axes                            
xlabel('x (m)');           %  label the x-axis
ylabel('height (m)');             %  label the y-axis
title('Ball flying through the air');     %  Give the plot a title at the top

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Set up detail plot ranges....
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

detailminx = 4.;   
detailminy = 0.;
detailmaxx = 4.5;
detailmaxy = 0.5;

figure(2);
axis([detailminx,detailmaxx,detailminy,detailmaxy]);  %  set limits on axes                            
xlabel('x (m)');           %  label the x-axis
ylabel('height (m)');             %  label the y-axis
title('Ball flying through the air [Detail]');     %  Give the plot a title at the top
